import * as MC from "@minecraft/server";
/** Return a promise which will be resolved after *at least* a given
 * fractional number of seconds have elapsed. The promise will never be
 * rejected.
 */
export function delay(secs) {
    return new Promise((resolve) => {
        setTimeout(resolve, Math.floor(secs * 1000));
    });
}
const timeoutMap = new Map();
let nextTimeoutID = 0;
let runId = null;
function mkHandler(arg0, ...args) {
    return typeof arg0 === "string"
        ? () => eval(arg0)
        : arg0.bind(null, ...args);
}
function onWorldTick() {
    const now = Date.now();
    for (const [tid, t] of timeoutMap) {
        if (t.startedAt + t.delay < now) {
            /* The handler might actually be an async function and it may
             * have been rejected. Catch the exception if it's the case. It
             * might also be a regular function and throws. */
            try {
                const ret = t.handler();
                Promise.resolve(ret).catch(e => console.error(e));
            }
            catch (e) {
                console.error(e);
            }
            if (t.repeat) {
                t.startedAt = now;
            }
            else {
                timeoutMap.delete(tid);
            }
        }
    }
    if (timeoutMap.size == 0) {
        unlistenOnTicks();
    }
}
function listenOnTicks() {
    if (runId == null) {
        runId = MC.system.runInterval(onWorldTick, 1);
    }
}
function unlistenOnTicks() {
    if (runId != null) {
        MC.system.clearRun(runId);
        runId = null;
    }
}
export function setTimeout(arg0, delay, ...args) {
    const tid = ++nextTimeoutID;
    timeoutMap.set(tid, {
        handler: mkHandler(arg0, ...args),
        startedAt: Date.now(),
        delay: delay || 0,
        repeat: false
    });
    listenOnTicks();
    return tid;
}
export function setInterval(arg0, delay, ...args) {
    const tid = ++nextTimeoutID;
    timeoutMap.set(tid, {
        handler: mkHandler(arg0, ...args),
        startedAt: Date.now(),
        delay: delay || 0,
        repeat: true
    });
    listenOnTicks();
    return tid;
}
/** A low-level implementation of the standard clearTimeout() function.
 */
export function clearTimeout(tid) {
    timeoutMap.delete(tid);
    if (timeoutMap.size == 0) {
        unlistenOnTicks();
    }
}
/** A low-level implementation of the standard clearInterval() function.
 */
export function clearInterval(tid) {
    clearTimeout(tid);
}
//# sourceMappingURL=delay.js.map
