/** A mixin for objects that have dynamic properties, such as Entity or
 * World.
 */
export function HasDynamicProperties(base) {
    class HasDynamicProperties extends base {
        getDynamicProperty(identifier, ty) {
            const prop = this.raw.getDynamicProperty(identifier);
            if (ty == null) {
                // No constraints are requested on the type of the property.
                return prop;
            }
            else {
                const expectedType = ty.substring(0, ty.length - 1);
                const actualType = typeof prop;
                if (prop === undefined) {
                    if (ty.endsWith("?")) {
                        // It can also be undefined.
                        return prop;
                    }
                    else {
                        throw new TypeError(`Dynamic property \`${identifier}' is expected to be a ${expectedType} but is actually undefined`);
                    }
                }
                else if (expectedType === "Vector3" && actualType === "object") {
                    return prop;
                }
                else if (expectedType === actualType) {
                    return prop;
                }
                else {
                    throw new TypeError(`Dynamic property \`${identifier} is expected to be a ${expectedType} but is actually a ${actualType}`);
                }
            }
        }
        setDynamicProperty(identifier, value) {
            this.raw.setDynamicProperty(identifier, value);
        }
    }
    return HasDynamicProperties;
}
//# sourceMappingURL=dynamic-props.js.map
