import { Container } from "../container.js";
import * as I from "../inspect.js";
import * as PP from "../pprint.js";
export class EntityInventory extends Container {
    #rawInv;
    static typeId = "minecraft:inventory";
    /// @internal
    constructor(rawInv) {
        if (rawInv.container)
            super(rawInv.container);
        else
            throw new Error(`This entity does not have a valid container`);
        this.#rawInv = rawInv;
    }
    get typeId() {
        return this.#rawInv.typeId;
    }
    get isValid() {
        return this.#rawInv.isValid();
    }
    get additionalSlotsPerStrength() {
        return this.#rawInv.additionalSlotsPerStrength;
    }
    get canBeSiphonedFrom() {
        return this.#rawInv.canBeSiphonedFrom;
    }
    get containerType() {
        return this.#rawInv.containerType;
    }
    get "private"() {
        return this.#rawInv.private;
    }
    get restrictToOwner() {
        return this.#rawInv.restrictToOwner;
    }
    /// @internal
    [I.customInspectSymbol](inspect, stylise) {
        const obj = {
            containerType: this.containerType,
        };
        // Now we want to display "this" as Container instead of
        // EntityInventory, so that inspecting obj.container will invoke
        // super[I.customInspectSymbol]. This is super hacky but I can't
        // think of any better ways.
        // @ts-ignore: TypeScript doesn't like super[key].
        const getSuperProp = (key) => super[key];
        obj.container = new Proxy(this, {
            "get"(self, key) {
                const prop = getSuperProp(key);
                if (typeof prop === "function")
                    return prop.bind(self);
                else
                    return prop;
            }
        });
        if (this.additionalSlotsPerStrength > 0)
            obj.additionalSlotsPerStrength = this.additionalSlotsPerStrength;
        if (this.canBeSiphonedFrom)
            obj.canBeSiphonedFrom = true;
        if (this.private)
            obj.private = true;
        if (this.restrictToOwner)
            obj.restrictToOwner = true;
        return PP.spaceCat(stylise(PP.text("EntityInventory"), I.TokenType.Class), inspect(obj));
    }
}
//# sourceMappingURL=inventory.js.map
