import { EnchantmentType } from "@minecraft/server";
import * as I from "../inspect.js";
import * as PP from "../pprint.js";
import * as MC from "@minecraft/server";
export { EnchantmentType };
export class ItemEnchantments {
    /// @internal
    raw;
    static typeId = "minecraft:enchantable";
    static "type"(enchantmentId) {
        return MC.EnchantmentTypes.get(enchantmentId);
    }
    /// @internal
    constructor(raw) {
        this.raw = raw;
    }
    get size() {
        // FIXME: Remove this glue code when the API is updated to 1.9.0.
        if (this.raw && "enchantments" in this.raw) {
            // @ts-ignore
            const list = this.raw.enchantments;
            let length = 0;
            for (const _ench of list) {
                length++;
            }
            return length;
        }
        return this.raw ? this.raw.getEnchantments().length : 0;
    }
    get [Symbol.toStringTag]() {
        return "ItemEnchantments";
    }
    [Symbol.iterator]() {
        return this.entries();
    }
    canSet(type, level) {
        // FIXME: Remove this glue code when the API is updated to 1.9.0.
        if (this.raw && "enchantments" in this.raw) {
            // @ts-ignore
            return this.raw.enchantments.canAddEnchantment(new MC.Enchantment(type, level));
        }
        return this.raw
            ? this.raw.canAddEnchantment({ type, level })
            : false;
    }
    clear() {
        if (this.raw) {
            this.raw.removeAllEnchantments();
        }
    }
    delete(type) {
        if (this.has(type)) {
            // FIXME: Remove this glue code when the API is updated to 1.9.0.
            if (this.raw && "enchantments" in this.raw) {
                // @ts-ignore
                this.raw.enchantments.removeEnchantment(type);
                return true;
            }
            this.raw.removeEnchantment(type);
            return true;
        }
        else {
            return false;
        }
    }
    *entries() {
        if (this.raw) {
            // FIXME: Remove this glue code when the API is updated to 1.9.0.
            if (this.raw && "enchantments" in this.raw) {
                // @ts-ignore
                for (const ench of this.raw.enchantments) {
                    yield [ench.type, ench.level];
                }
                return;
            }
            for (const ench of this.raw.getEnchantments()) {
                yield [ench.type, ench.level];
            }
        }
    }
    forEach(f, thisArg) {
        const boundF = f.bind(thisArg);
        for (const [type, level] of this) {
            boundF(level, type, this);
        }
    }
    "get"(type) {
        if (this.raw) {
            // FIXME: Remove this glue code when the API is updated to 1.9.0.
            if ("enchantments" in this.raw) {
                // EnchantmentList.prototype.getEnchantment() isn't
                // callable in read-only mode but [@@iterator] is callable,
                // although the documentation states that neither are
                // callable. Weird.
                try {
                    // @ts-ignore
                    const ench = this.raw.enchantments.getEnchantment(type);
                    return ench ? ench.level : undefined;
                }
                catch (e) {
                    if (typeof type === "string") {
                        for (const [type1, level] of this) {
                            if (type === type1.id)
                                return level;
                        }
                    }
                    else {
                        for (const [type1, level] of this) {
                            if (type.id === type1.id)
                                return level;
                        }
                    }
                    return undefined;
                }
            }
            const ench = this.raw.getEnchantment(type);
            return ench ? ench.level : undefined;
        }
        else {
            return undefined;
        }
    }
    has(type) {
        if (this.raw) {
            // FIXME: Remove this glue code when the API is updated to 1.9.0.
            if ("enchantments" in this.raw) {
                // @ts-ignore
                return this.raw.enchantments.hasEnchantment(type) > 0;
            }
            return this.raw.hasEnchantment(type);
        }
        else {
            return false;
        }
    }
    *keys() {
        if (this.raw) {
            // FIXME: Remove this glue code when the API is updated to 1.9.0.
            if (this.raw && "enchantments" in this.raw) {
                // @ts-ignore
                for (const ench of this.raw.enchantments) {
                    yield ench.type;
                }
                return;
            }
            for (const ench of this.raw.getEnchantments()) {
                yield ench.type;
            }
        }
    }
    "set"(type, level) {
        if (this.raw) {
            // FIXME: Remove this glue code when the API is updated to 1.9.0.
            if ("enchantments" in this.raw) {
                // @ts-ignore
                this.raw.enchantments.addEnchantment(new MC.Enchantment(type, level));
                return this;
            }
            this.raw.addEnchantment({ type, level });
            return this;
        }
        else {
            throw new TypeError("this item can have no enchantments");
        }
    }
    *values() {
        if (this.raw) {
            // FIXME: Remove this glue code when the API is updated to 1.9.0.
            if (this.raw && "enchantments" in this.raw) {
                // @ts-ignore
                for (const ench of this.raw.enchantments) {
                    yield ench.level;
                }
                return;
            }
            for (const ench of this.raw.getEnchantments()) {
                yield ench.level;
            }
        }
    }
    /// @internal
    [I.customInspectSymbol](inspect, stylise, opts) {
        /* [ItemEnchantments] {
         *     "silk_touch",
         *     "unbreaking" 1 (max: 3),
         *     "efficiency" 5 (max: 5)
         * }
         */
        const prefix = stylise(PP.text("ItemEnchantments"), I.TokenType.Class);
        const elems = [];
        for (const [type, level] of this) {
            const docs = [];
            docs.push(inspect(type.id));
            if (level > 1 || type.maxLevel > 1) {
                docs.push(inspect(level));
                docs.push(stylise(PP.parens(PP.spaceCat(PP.text("max:"), PP.number(type.maxLevel))), I.TokenType.Tag));
            }
            elems.push(PP.hsep(docs));
        }
        if (elems.length > 0)
            // If the entire object fits the line, print it in a single
            // line. Otherwise break lines for each enchantments.
            return PP.spaceCat(prefix, PP.group(PP.lineCat(PP.nest(opts.indentationWidth, PP.lineCat(PP.lbrace, PP.vsep(PP.punctuate(PP.comma, elems)))), PP.rbrace)));
        else
            return PP.spaceCat(prefix, PP.braces(PP.empty));
    }
}
I.overrideInspector(EnchantmentType, function (inspect, stylise) {
    return PP.hsep([
        stylise(PP.brackets(PP.text("EnchantmentType")), I.TokenType.Tag),
        stylise(PP.brackets(PP.text(`max: ${this.maxLevel}`)), I.TokenType.Tag),
        inspect(this.id)
    ]);
});
//# sourceMappingURL=enchantments.js.map
